"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const core_1 = require("@oclif/core");
const command_types_1 = require("../types/command.types");
const auth_service_1 = require("../services/auth.service");
const config_service_1 = require("../services/config.service");
const validation_service_1 = require("../services/validation.service");
const cli_utils_1 = require("../utils/cli.utils");
const errors_utils_1 = require("../utils/errors.utils");
const sdk_manager_service_1 = require("../services/sdk-manager.service");
const drive_database_manager_service_1 = require("../services/database/drive-database-manager.service");
class Login extends core_1.Command {
    static args = {};
    static description = 'Logs into an Internxt account. If the account is two-factor protected, then an extra code will be required.';
    static aliases = [];
    static examples = ['<%= config.bin %> <%= command.id %>'];
    static flags = {
        ...cli_utils_1.CLIUtils.CommonFlags,
        email: core_1.Flags.string({
            char: 'e',
            aliases: ['mail'],
            env: 'INXT_EMAIL',
            description: 'The email to log in',
            required: false,
        }),
        password: core_1.Flags.string({
            char: 'p',
            aliases: ['pass'],
            env: 'INXT_PASSWORD',
            description: 'The plain password to log in',
            required: false,
        }),
        twofactor: core_1.Flags.string({
            char: 'w',
            aliases: ['two', 'two-factor'],
            env: 'INXT_TWOFACTORCODE',
            description: 'The two factor auth code (only needed if the account is two-factor protected)',
            required: false,
            helpValue: '123456',
        }),
    };
    static enableJsonFlag = true;
    run = async () => {
        const { flags } = await this.parse(Login);
        const nonInteractive = flags['non-interactive'];
        const email = await this.getEmail(flags['email'], nonInteractive);
        const password = await this.getPassword(flags['password'], nonInteractive);
        const is2FANeeded = await auth_service_1.AuthService.instance.is2FANeeded(email);
        let twoFactorCode;
        if (is2FANeeded) {
            twoFactorCode = await this.getTwoFactorCode(flags['twofactor'], nonInteractive);
        }
        const loginCredentials = await auth_service_1.AuthService.instance.doLogin(email, password, twoFactorCode);
        sdk_manager_service_1.SdkManager.init({
            token: loginCredentials.token,
            newToken: loginCredentials.newToken,
        });
        await config_service_1.ConfigService.instance.saveUser(loginCredentials);
        await drive_database_manager_service_1.DriveDatabaseManager.init();
        await drive_database_manager_service_1.DriveDatabaseManager.clean();
        const message = `Succesfully logged in to: ${loginCredentials.user.email}`;
        cli_utils_1.CLIUtils.success(this.log.bind(this), message);
        return {
            success: true,
            message,
            login: loginCredentials,
        };
    };
    catch = async (error) => {
        errors_utils_1.ErrorUtils.report(this.error.bind(this), error, { command: this.id });
        cli_utils_1.CLIUtils.error(this.log.bind(this), error.message);
        this.exit(1);
    };
    getEmail = async (emailFlag, nonInteractive) => {
        const email = await cli_utils_1.CLIUtils.getValueFromFlag({
            value: emailFlag,
            name: Login.flags['email'].name,
        }, {
            nonInteractive,
            prompt: {
                message: 'What is your email?',
                options: { type: 'input' },
            },
        }, {
            validate: validation_service_1.ValidationService.instance.validateEmail,
            error: new command_types_1.NotValidEmailError(),
        }, this.log.bind(this));
        return email;
    };
    getPassword = async (passwordFlag, nonInteractive) => {
        const password = await cli_utils_1.CLIUtils.getValueFromFlag({
            value: passwordFlag,
            name: Login.flags['password'].name,
        }, {
            nonInteractive,
            prompt: {
                message: 'What is your password?',
                options: { type: 'password' },
            },
        }, {
            validate: validation_service_1.ValidationService.instance.validateStringIsNotEmpty,
            error: new command_types_1.EmptyPasswordError(),
        }, this.log.bind(this));
        return password;
    };
    getTwoFactorCode = async (twoFactorFlag, nonInteractive) => {
        const twoFactor = await cli_utils_1.CLIUtils.getValueFromFlag({
            value: twoFactorFlag,
            name: Login.flags['twofactor'].name,
        }, {
            nonInteractive,
            prompt: {
                message: 'What is your two-factor token?',
                options: { type: 'mask' },
            },
        }, {
            validate: validation_service_1.ValidationService.instance.validate2FA,
            error: new command_types_1.NotValidTwoFactorCodeError(),
        }, this.log.bind(this));
        return twoFactor;
    };
}
exports.default = Login;
