"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const core_1 = require("@oclif/core");
const config_service_1 = require("../services/config.service");
const drive_folder_service_1 = require("../services/drive/drive-folder.service");
const cli_utils_1 = require("../utils/cli.utils");
const command_types_1 = require("../types/command.types");
const validation_service_1 = require("../services/validation.service");
const errors_utils_1 = require("../utils/errors.utils");
class TrashRestoreFolder extends core_1.Command {
    static args = {};
    static description = 'Restore a trashed folder into a destination folder.';
    static aliases = ['trash:restore:folder'];
    static examples = ['<%= config.bin %> <%= command.id %>'];
    static flags = {
        ...cli_utils_1.CLIUtils.CommonFlags,
        id: core_1.Flags.string({
            char: 'i',
            description: 'The folder id to be restored from the trash.',
            required: false,
        }),
        destination: core_1.Flags.string({
            char: 'd',
            description: 'The folder id where the folder is going to be restored. Leave empty for the root folder.',
            required: false,
            parse: cli_utils_1.CLIUtils.parseEmpty,
        }),
    };
    static enableJsonFlag = true;
    run = async () => {
        const { flags } = await this.parse(TrashRestoreFolder);
        const nonInteractive = flags['non-interactive'];
        const userCredentials = await config_service_1.ConfigService.instance.readUser();
        if (!userCredentials)
            throw new command_types_1.MissingCredentialsError();
        const folderUuid = await this.getFolderUuid(flags['id'], nonInteractive);
        let destinationFolderUuid = await this.getDestinationFolderUuid(flags['destination'], nonInteractive);
        if (destinationFolderUuid.trim().length === 0) {
            destinationFolderUuid = userCredentials.user.rootFolderId;
        }
        const folder = await drive_folder_service_1.DriveFolderService.instance.moveFolder({ folderUuid, destinationFolderUuid });
        const message = `Folder restored successfully to: ${destinationFolderUuid}`;
        cli_utils_1.CLIUtils.success(this.log.bind(this), message);
        return { success: true, message, folder };
    };
    catch = async (error) => {
        errors_utils_1.ErrorUtils.report(this.error.bind(this), error, { command: this.id });
        cli_utils_1.CLIUtils.error(this.log.bind(this), error.message);
        this.exit(1);
    };
    getFolderUuid = async (folderUuidFlag, nonInteractive) => {
        const folderUuid = await cli_utils_1.CLIUtils.getValueFromFlag({
            value: folderUuidFlag,
            name: TrashRestoreFolder.flags['id'].name,
        }, {
            nonInteractive,
            prompt: {
                message: 'What is the folder id you want to restore?',
                options: { type: 'input' },
            },
        }, {
            validate: validation_service_1.ValidationService.instance.validateUUIDv4,
            error: new command_types_1.NotValidFolderUuidError(),
        }, this.log.bind(this));
        return folderUuid;
    };
    getDestinationFolderUuid = async (destinationFolderUuidFlag, nonInteractive) => {
        const destinationFolderUuid = await cli_utils_1.CLIUtils.getValueFromFlag({
            value: destinationFolderUuidFlag,
            name: TrashRestoreFolder.flags['destination'].name,
        }, {
            nonInteractive,
            prompt: {
                message: 'What is the destination folder id? (leave empty for the root folder)',
                options: { type: 'input' },
            },
        }, {
            validate: validation_service_1.ValidationService.instance.validateUUIDv4,
            error: new command_types_1.NotValidFolderUuidError(),
            canBeEmpty: true,
        }, this.log.bind(this));
        return destinationFolderUuid;
    };
}
exports.default = TrashRestoreFolder;
