"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const core_1 = require("@oclif/core");
const promises_1 = require("node:fs/promises");
const node_fs_1 = require("node:fs");
const network_facade_service_1 = require("../services/network/network-facade.service");
const sdk_manager_service_1 = require("../services/sdk-manager.service");
const auth_service_1 = require("../services/auth.service");
const cli_utils_1 = require("../utils/cli.utils");
const config_service_1 = require("../services/config.service");
const node_path_1 = __importDefault(require("node:path"));
const drive_file_service_1 = require("../services/drive/drive-file.service");
const crypto_service_1 = require("../services/crypto.service");
const download_service_1 = require("../services/network/download.service");
const errors_utils_1 = require("../utils/errors.utils");
const command_types_1 = require("../types/command.types");
const validation_service_1 = require("../services/validation.service");
const types_1 = require("@internxt/sdk/dist/drive/storage/types");
const thumbnail_service_1 = require("../services/thumbnail.service");
const stream_utils_1 = require("../utils/stream.utils");
const thumbnail_utils_1 = require("../utils/thumbnail.utils");
const inxt_js_1 = require("@internxt/inxt-js");
class UploadFile extends core_1.Command {
    static args = {};
    static description = 'Upload a file to Internxt Drive';
    static aliases = ['upload:file'];
    static examples = ['<%= config.bin %> <%= command.id %>'];
    static flags = {
        ...cli_utils_1.CLIUtils.CommonFlags,
        file: core_1.Flags.string({
            char: 'f',
            description: 'The path to the file on your system.',
            required: false,
        }),
        destination: core_1.Flags.string({
            char: 'i',
            description: 'The folder id where the file is going to be uploaded to. Leave empty for the root folder.',
            required: false,
            parse: cli_utils_1.CLIUtils.parseEmpty,
        }),
    };
    static enableJsonFlag = true;
    run = async () => {
        const { flags } = await this.parse(UploadFile);
        const nonInteractive = flags['non-interactive'];
        const userCredentials = await config_service_1.ConfigService.instance.readUser();
        if (!userCredentials)
            throw new command_types_1.MissingCredentialsError();
        const filePath = await this.getFilePath(flags['file'], nonInteractive);
        const stats = await (0, promises_1.stat)(filePath);
        if (!stats.size) {
            throw new Error('The file is empty. Uploading empty files is not allowed.');
        }
        const fileInfo = node_path_1.default.parse(filePath);
        const fileType = fileInfo.ext.replaceAll('.', '');
        let destinationFolderUuid = await this.getDestinationFolderUuid(flags['destination'], nonInteractive);
        if (destinationFolderUuid.trim().length === 0) {
            destinationFolderUuid = userCredentials.user.rootFolderId;
        }
        cli_utils_1.CLIUtils.doing('Preparing Network');
        const { user } = await auth_service_1.AuthService.instance.getAuthDetails();
        const networkModule = sdk_manager_service_1.SdkManager.instance.getNetwork({
            user: user.bridgeUser,
            pass: user.userId,
        });
        const environment = new inxt_js_1.Environment({
            bridgeUser: user.bridgeUser,
            bridgePass: user.userId,
            bridgeUrl: config_service_1.ConfigService.instance.get('NETWORK_URL'),
            encryptionKey: user.mnemonic,
        });
        const networkFacade = new network_facade_service_1.NetworkFacade(networkModule, environment, download_service_1.DownloadService.instance, crypto_service_1.CryptoService.instance);
        cli_utils_1.CLIUtils.done();
        const readStream = (0, node_fs_1.createReadStream)(filePath);
        const timer = cli_utils_1.CLIUtils.timer();
        const progressBar = cli_utils_1.CLIUtils.progress({
            format: 'Uploading file [{bar}] {percentage}%',
            linewrap: true,
        });
        progressBar.start(100, 0);
        let bufferStream;
        let fileStream = readStream;
        const isThumbnailable = (0, thumbnail_utils_1.isFileThumbnailable)(fileType);
        if (isThumbnailable) {
            bufferStream = new stream_utils_1.BufferStream();
            fileStream = readStream.pipe(bufferStream);
        }
        const progressCallback = (progress) => {
            progressBar.update(progress * 100 * 0.99);
        };
        const fileId = await new Promise((resolve, reject) => {
            const state = networkFacade.uploadFile(fileStream, stats.size, user.bucket, (err, res) => {
                if (err) {
                    return reject(err);
                }
                resolve(res);
            }, progressCallback);
            process.on('SIGINT', () => {
                state.stop();
                process.exit(1);
            });
        });
        const createdDriveFile = await drive_file_service_1.DriveFileService.instance.createFile({
            plain_name: fileInfo.name,
            type: fileType,
            size: stats.size,
            folder_id: destinationFolderUuid,
            id: fileId,
            bucket: user.bucket,
            encrypt_version: types_1.EncryptionVersion.Aes03,
            name: '',
        });
        try {
            if (isThumbnailable && bufferStream) {
                const thumbnailBuffer = bufferStream.getBuffer();
                if (thumbnailBuffer) {
                    await thumbnail_service_1.ThumbnailService.instance.uploadThumbnail(thumbnailBuffer, fileType, user.bucket, createdDriveFile.id, networkFacade);
                }
            }
        }
        catch (error) {
            errors_utils_1.ErrorUtils.report(this.error.bind(this), error, { command: this.id });
        }
        progressBar.update(100);
        progressBar.stop();
        const uploadTime = timer.stop();
        this.log('\n');
        const message = `File uploaded in ${uploadTime}ms, view it at ${config_service_1.ConfigService.instance.get('DRIVE_URL')}/file/${createdDriveFile.uuid}`;
        cli_utils_1.CLIUtils.success(this.log.bind(this), message);
        return { success: true, message, file: createdDriveFile };
    };
    catch = async (error) => {
        errors_utils_1.ErrorUtils.report(this.error.bind(this), error, { command: this.id });
        cli_utils_1.CLIUtils.error(this.log.bind(this), error.message);
        this.exit(1);
    };
    getDestinationFolderUuid = async (destinationFolderUuidFlag, nonInteractive) => {
        const destinationFolderUuid = await cli_utils_1.CLIUtils.getValueFromFlag({
            value: destinationFolderUuidFlag,
            name: UploadFile.flags['destination'].name,
        }, {
            nonInteractive,
            prompt: {
                message: 'What is the destination folder id? (leave empty for the root folder)',
                options: { type: 'input' },
            },
        }, {
            validate: validation_service_1.ValidationService.instance.validateUUIDv4,
            error: new command_types_1.NotValidFolderUuidError(),
            canBeEmpty: true,
        }, this.log.bind(this));
        return destinationFolderUuid;
    };
    getFilePath = async (fileFlag, nonInteractive) => {
        const filePath = await cli_utils_1.CLIUtils.getValueFromFlag({
            value: fileFlag,
            name: UploadFile.flags['file'].name,
        }, {
            nonInteractive,
            prompt: {
                message: 'What is the path to the file on your computer?',
                options: { type: 'input' },
            },
        }, {
            validate: validation_service_1.ValidationService.instance.validateFileExists,
            error: new command_types_1.NotValidDirectoryError(),
        }, this.log.bind(this));
        return filePath;
    };
}
exports.default = UploadFile;
