"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const core_1 = require("@oclif/core");
const pm2_utils_1 = require("../utils/pm2.utils");
const cli_utils_1 = require("../utils/cli.utils");
const config_service_1 = require("../services/config.service");
const drive_database_manager_service_1 = require("../services/database/drive-database-manager.service");
const errors_utils_1 = require("../utils/errors.utils");
const auth_service_1 = require("../services/auth.service");
class Webdav extends core_1.Command {
    static args = {
        action: core_1.Args.string({
            required: true,
            options: ['enable', 'disable', 'restart', 'status'],
        }),
    };
    static description = 'Enable, disable, restart or get the status of the Internxt CLI WebDav server';
    static aliases = [];
    static examples = [
        '<%= config.bin %> <%= command.id %> enable',
        '<%= config.bin %> <%= command.id %> disable',
        '<%= config.bin %> <%= command.id %> restart',
        '<%= config.bin %> <%= command.id %> status',
    ];
    static flags = {};
    static enableJsonFlag = true;
    run = async () => {
        const { args } = await this.parse(Webdav);
        let message = '';
        let success = true;
        await pm2_utils_1.PM2Utils.connect();
        switch (args.action) {
            case 'enable': {
                await auth_service_1.AuthService.instance.getAuthDetails();
                message = await this.enableWebDav();
                break;
            }
            case 'disable': {
                message = await this.disableWebDav();
                break;
            }
            case 'restart': {
                await auth_service_1.AuthService.instance.getAuthDetails();
                message = await this.restartWebDav();
                break;
            }
            case 'status': {
                await auth_service_1.AuthService.instance.getAuthDetails();
                message = await this.webDAVStatus();
                break;
            }
            default: {
                success = false;
                message = `Expected one of this command actions: ${Webdav.args.action.options}`;
                break;
            }
        }
        pm2_utils_1.PM2Utils.disconnect();
        return { success, message, action: args.action };
    };
    catch = async (error) => {
        errors_utils_1.ErrorUtils.report(this.error.bind(this), error, { command: this.id });
        if (error instanceof Error) {
            cli_utils_1.CLIUtils.error(this.log.bind(this), error.message);
        }
        else {
            cli_utils_1.CLIUtils.error(this.log.bind(this), JSON.stringify(error));
        }
        this.exit(1);
    };
    enableWebDav = async () => {
        cli_utils_1.CLIUtils.doing('Starting Internxt WebDav server...');
        await drive_database_manager_service_1.DriveDatabaseManager.clean();
        await pm2_utils_1.PM2Utils.killWebDavServer();
        await pm2_utils_1.PM2Utils.startWebDavServer();
        cli_utils_1.CLIUtils.done();
        const { status } = await pm2_utils_1.PM2Utils.webdavServerStatus();
        const webdavConfigs = await config_service_1.ConfigService.instance.readWebdavConfig();
        if (status === 'online') {
            const message = `Internxt WebDav server started successfully at ${webdavConfigs.protocol}://${config_service_1.ConfigService.WEBDAV_LOCAL_URL}:${webdavConfigs.port}`;
            cli_utils_1.CLIUtils.log(this.log.bind(this), `\nWebDav server status: ${core_1.ux.colorize('green', 'online')}\n`);
            cli_utils_1.CLIUtils.success(this.log.bind(this), message);
            cli_utils_1.CLIUtils.log(this.log.bind(this), `\n[If the above URL is not working, the WebDAV server can be accessed directly via your localhost IP at: ${webdavConfigs.protocol}://127.0.0.1:${webdavConfigs.port} ]\n`);
            return message;
        }
        else {
            const message = `WebDav server status: ${core_1.ux.colorize('red', status)}`;
            cli_utils_1.CLIUtils.log(this.log.bind(this), message);
            return message;
        }
    };
    disableWebDav = async () => {
        cli_utils_1.CLIUtils.doing('Stopping Internxt WebDav server...');
        await pm2_utils_1.PM2Utils.killWebDavServer();
        cli_utils_1.CLIUtils.done();
        const message = 'Internxt WebDav server stopped successfully';
        cli_utils_1.CLIUtils.success(this.log.bind(this), message);
        return message;
    };
    restartWebDav = async () => {
        cli_utils_1.CLIUtils.doing('Restarting Internxt WebDav server...');
        await drive_database_manager_service_1.DriveDatabaseManager.clean();
        const { status } = await pm2_utils_1.PM2Utils.webdavServerStatus();
        if (status === 'online') {
            await pm2_utils_1.PM2Utils.killWebDavServer();
            await pm2_utils_1.PM2Utils.startWebDavServer();
            cli_utils_1.CLIUtils.done();
            const message = 'Internxt WebDav server restarted successfully';
            cli_utils_1.CLIUtils.success(this.log.bind(this), message);
            return message;
        }
        else {
            cli_utils_1.CLIUtils.done();
            const message = 'Internxt WebDav server is not running, it wont be restarted';
            cli_utils_1.CLIUtils.warning(this.log.bind(this), message);
            return message;
        }
    };
    webDAVStatus = async () => {
        const { status } = await pm2_utils_1.PM2Utils.webdavServerStatus();
        const message = `Internxt WebDAV server status: ${status}`;
        cli_utils_1.CLIUtils.log(this.log.bind(this), message);
        return message;
    };
}
exports.default = Webdav;
