"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.AuthService = void 0;
const sdk_manager_service_1 = require("./sdk-manager.service");
const keys_service_1 = require("./keys.service");
const crypto_service_1 = require("./crypto.service");
const config_service_1 = require("./config.service");
const command_types_1 = require("../types/command.types");
const validation_service_1 = require("./validation.service");
class AuthService {
    static instance = new AuthService();
    doLogin = async (email, password, twoFactorCode) => {
        const authClient = sdk_manager_service_1.SdkManager.instance.getAuth();
        const loginDetails = {
            email: email.toLowerCase(),
            password: password,
            tfaCode: twoFactorCode,
        };
        const data = await authClient.login(loginDetails, crypto_service_1.CryptoService.cryptoProvider);
        const { user, token, newToken } = data;
        const { privateKey, publicKey } = user;
        const plainPrivateKeyInBase64 = privateKey
            ? Buffer.from(keys_service_1.KeysService.instance.decryptPrivateKey(privateKey, password)).toString('base64')
            : '';
        if (privateKey) {
            await keys_service_1.KeysService.instance.assertPrivateKeyIsValid(privateKey, password);
            await keys_service_1.KeysService.instance.assertValidateKeys(Buffer.from(plainPrivateKeyInBase64, 'base64').toString(), Buffer.from(publicKey, 'base64').toString());
        }
        const clearMnemonic = crypto_service_1.CryptoService.instance.decryptTextWithKey(user.mnemonic, password);
        const clearUser = {
            ...user,
            mnemonic: clearMnemonic,
            privateKey: plainPrivateKeyInBase64,
        };
        return {
            user: clearUser,
            token: token,
            newToken: newToken,
            lastLoggedInAt: new Date().toISOString(),
            lastTokenRefreshAt: new Date().toISOString(),
        };
    };
    is2FANeeded = async (email) => {
        const authClient = sdk_manager_service_1.SdkManager.instance.getAuth();
        const securityDetails = await authClient.securityDetails(email).catch((error) => {
            throw new Error(error.message ?? 'Login error');
        });
        return securityDetails.tfaEnabled;
    };
    getAuthDetails = async () => {
        let loginCreds = await config_service_1.ConfigService.instance.readUser();
        if (!(loginCreds?.newToken && loginCreds?.token && loginCreds?.user?.mnemonic)) {
            throw new command_types_1.MissingCredentialsError();
        }
        const oldTokenDetails = validation_service_1.ValidationService.instance.validateTokenAndCheckExpiration(loginCreds.token);
        const newTokenDetails = validation_service_1.ValidationService.instance.validateTokenAndCheckExpiration(loginCreds.newToken);
        const isValidMnemonic = validation_service_1.ValidationService.instance.validateMnemonic(loginCreds.user.mnemonic);
        if (!(oldTokenDetails.isValid && newTokenDetails.isValid && isValidMnemonic)) {
            throw new command_types_1.InvalidCredentialsError();
        }
        if (oldTokenDetails.expiration.expired || newTokenDetails.expiration.expired) {
            throw new command_types_1.ExpiredCredentialsError();
        }
        const refreshOldToken = !oldTokenDetails.expiration.expired && oldTokenDetails.expiration.refreshRequired;
        const refreshNewToken = !newTokenDetails.expiration.expired && newTokenDetails.expiration.refreshRequired;
        if (refreshOldToken || refreshNewToken) {
            loginCreds = await this.refreshUserTokens(loginCreds);
        }
        return loginCreds;
    };
    refreshUserTokens = async (oldCreds) => {
        const usersClient = sdk_manager_service_1.SdkManager.instance.getUsers(true);
        const newCreds = await usersClient.getUserData({ userUuid: oldCreds.user.uuid });
        const loginCreds = {
            user: {
                ...newCreds.user,
                mnemonic: oldCreds.user.mnemonic,
                privateKey: oldCreds.user.privateKey,
            },
            token: newCreds.oldToken,
            newToken: newCreds.newToken,
            lastLoggedInAt: oldCreds.lastLoggedInAt,
            lastTokenRefreshAt: new Date().toISOString(),
        };
        return loginCreds;
    };
}
exports.AuthService = AuthService;
