"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.DriveDatabaseManager = void 0;
const sequelize_typescript_1 = require("sequelize-typescript");
const webdav_utils_1 = require("../../utils/webdav.utils");
const config_service_1 = require("../config.service");
const drive_file_repository_1 = require("./drive-file/drive-file.repository");
const drive_folder_repository_1 = require("./drive-folder/drive-folder.repository");
const drive_file_model_1 = __importDefault(require("./drive-file/drive-file.model"));
const drive_folder_model_1 = __importDefault(require("./drive-folder/drive-folder.model"));
class DriveDatabaseManager {
    driveFileRepository;
    driveFolderRepository;
    static sequelize = new sequelize_typescript_1.Sequelize({
        dialect: 'sqlite',
        storage: config_service_1.ConfigService.DRIVE_SQLITE_FILE,
        models: [drive_file_model_1.default, drive_folder_model_1.default],
        logging: false,
    });
    constructor(driveFileRepository, driveFolderRepository) {
        this.driveFileRepository = driveFileRepository;
        this.driveFolderRepository = driveFolderRepository;
    }
    static init = async () => {
        await DriveDatabaseManager.sequelize.sync({ force: true });
    };
    static clean = async () => {
        await drive_file_repository_1.DriveFileRepository.clean();
        await drive_folder_repository_1.DriveFolderRepository.clean();
    };
    findFileByRelativePath = async (relativePath) => {
        const driveFile = await this.driveFileRepository.findByRelativePath(relativePath);
        return driveFile ?? null;
    };
    findFolderByRelativePath = async (relativePath) => {
        const folderRelativePath = relativePath.endsWith('/') ? relativePath : relativePath.concat('/');
        const driveFolder = await this.driveFolderRepository.findByRelativePath(folderRelativePath);
        return driveFolder ?? null;
    };
    createFolder = async (driveFolder, relativePath) => {
        await this.deleteFolderById(driveFolder.id);
        await this.deleteFolderByPath(relativePath);
        return await this.driveFolderRepository.createFolder(driveFolder, relativePath);
    };
    createFile = async (driveFile, relativePath) => {
        await this.deleteFileById(driveFile.id);
        await this.deleteFileByPath(relativePath);
        return await this.driveFileRepository.createFile(driveFile, relativePath);
    };
    buildRelativePathForFile = async (fileName, parentId) => {
        const parentFolder = await this.driveFolderRepository.findById(parentId ?? -1);
        if (!parentFolder) {
            return webdav_utils_1.WebDavUtils.joinURL('/', fileName);
        }
        const parentPath = await this.buildRelativePathForFile(parentFolder.name, parentFolder.parentId);
        return webdav_utils_1.WebDavUtils.joinURL(parentPath, fileName);
    };
    buildRelativePathForFolder = async (folderName, parentId) => {
        const parentFolder = await this.driveFolderRepository.findById(parentId ?? -1);
        if (!parentFolder) {
            return webdav_utils_1.WebDavUtils.joinURL('/', folderName, '/');
        }
        const parentPath = await this.buildRelativePathForFolder(parentFolder.name, parentFolder.parentId);
        return webdav_utils_1.WebDavUtils.joinURL(parentPath, folderName, '/');
    };
    deleteFileById = async (id) => {
        const existingObject = await this.driveFileRepository.findById(id);
        if (existingObject) {
            await this.driveFileRepository.deleteById(existingObject.id);
        }
    };
    deleteFolderById = async (id) => {
        const existingObject = await this.driveFolderRepository.findById(id);
        if (existingObject) {
            await this.driveFolderRepository.deleteById(existingObject.id);
        }
    };
    deleteFileByPath = async (relativePath) => {
        const existingPath = await this.driveFileRepository.findByRelativePath(relativePath);
        if (existingPath)
            await this.driveFileRepository.deleteById(existingPath.id);
    };
    deleteFolderByPath = async (relativePath) => {
        const existingPath = await this.driveFolderRepository.findByRelativePath(relativePath);
        if (existingPath) {
            await this.driveFolderRepository.deleteById(existingPath.id);
        }
    };
}
exports.DriveDatabaseManager = DriveDatabaseManager;
