"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.NoFlagProvidedError = exports.CLIUtils = void 0;
const core_1 = require("@oclif/core");
const cli_progress_1 = __importDefault(require("cli-progress"));
const tty_table_1 = __importDefault(require("tty-table"));
const inquirer_utils_1 = require("./inquirer.utils");
class CLIUtils {
    static clearPreviousLine = () => {
        process.stdout.write('\x1b[1A');
        process.stdout.clearLine(0);
    };
    static warning = (reporter, message) => {
        reporter(core_1.ux.colorize('#a67805', `⚠ Warning: ${message}`));
    };
    static error = (reporter, message) => {
        reporter(core_1.ux.colorize('red', `⚠ Error: ${message}`));
    };
    static doing = (message) => {
        core_1.ux.action.start(message, undefined, {});
    };
    static success = (reporter, message) => {
        reporter(core_1.ux.colorize('green', `✓ ${message}`));
    };
    static log = (reporter, message) => {
        reporter(`${message}`);
    };
    static done = () => {
        core_1.ux.action.stop(core_1.ux.colorize('green', 'done ✓'));
    };
    static progress = (opts) => {
        return new cli_progress_1.default.SingleBar({ noTTYOutput: Boolean(!process.stdin.isTTY), ...opts }, cli_progress_1.default.Presets.shades_classic);
    };
    static table = (reporter, header, rows) => {
        const table = (0, tty_table_1.default)(header, rows);
        reporter(table.render());
    };
    static generateTableHeaderFromType = () => {
        return Object.keys({}).map((key) => ({
            value: key,
            alias: key.charAt(0).toUpperCase() + key.slice(1),
        }));
    };
    static CommonFlags = {
        'non-interactive': core_1.Flags.boolean({
            char: 'x',
            env: 'INXT_NONINTERACTIVE',
            helpGroup: 'helper',
            description: 'Prevents the CLI from being interactive. When enabled, the CLI will not request input through the console and will throw errors directly.',
            required: false,
        }),
    };
    static getValueFromFlag = async (flag, command, validation, reporter) => {
        if (flag.value) {
            if (validation.canBeEmpty && flag.value.trim().length === 0) {
                return '';
            }
            const isValid = await validation.validate(flag.value);
            if (isValid) {
                return flag.value;
            }
            else if (command.nonInteractive) {
                throw validation.error;
            }
            else {
                CLIUtils.error(reporter, validation.error.message);
            }
        }
        if (command.nonInteractive) {
            throw new NoFlagProvidedError(flag.name);
        }
        else {
            const maxAttempts = command.maxAttempts ?? 3;
            return await CLIUtils.promptWithAttempts(command.prompt, maxAttempts, validation, reporter);
        }
    };
    static promptWithAttempts = async (prompt, maxAttempts, validation, reporter) => {
        let isValid = false;
        let currentAttempts = 0;
        let promptValue = '';
        do {
            promptValue = await inquirer_utils_1.InquirerUtils.prompt(prompt.message, prompt.options);
            if (validation.canBeEmpty) {
                if (!promptValue || promptValue.trim().length === 0) {
                    return '';
                }
            }
            isValid = await validation.validate(promptValue);
            if (!isValid) {
                currentAttempts++;
                if (currentAttempts < maxAttempts) {
                    CLIUtils.warning(reporter, validation.error.message + ', please type it again');
                }
            }
        } while (!isValid && currentAttempts < maxAttempts);
        if (!isValid) {
            throw validation.error;
        }
        else {
            return promptValue;
        }
    };
    static timer = () => {
        const start = new Date();
        return {
            stop: () => {
                const end = new Date();
                const time = end.getTime() - start.getTime();
                return time;
            },
        };
    };
    static parseEmpty = async (input) => (input.trim().length === 0 ? ' ' : input);
}
exports.CLIUtils = CLIUtils;
class NoFlagProvidedError extends Error {
    constructor(flag) {
        super(`No ${flag} flag has been provided`);
        Object.setPrototypeOf(this, NoFlagProvidedError.prototype);
    }
}
exports.NoFlagProvidedError = NoFlagProvidedError;
