"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.WebDavUtils = void 0;
const node_path_1 = __importDefault(require("node:path"));
const errors_utils_1 = require("./errors.utils");
const logger_utils_1 = require("./logger.utils");
class WebDavUtils {
    static joinURL(...pathComponents) {
        return node_path_1.default.posix.join(...pathComponents);
    }
    static removeHostFromURL(completeURL) {
        if (!completeURL.startsWith('/') && !/^https?:\/\//i.test(completeURL)) {
            completeURL = 'https://' + completeURL;
        }
        const parsedUrl = new URL(completeURL);
        let url = parsedUrl.href.replace(parsedUrl.origin + '/', '');
        if (!url.startsWith('/'))
            url = '/'.concat(url);
        return url;
    }
    static async getRequestedResource(urlObject) {
        let requestUrl;
        if (typeof urlObject === 'string') {
            requestUrl = urlObject;
        }
        else {
            requestUrl = urlObject.url;
        }
        const decodedUrl = decodeURIComponent(requestUrl).replaceAll('/./', '/');
        const parsedPath = node_path_1.default.parse(decodedUrl);
        let parentPath = node_path_1.default.dirname(decodedUrl);
        if (!parentPath.startsWith('/'))
            parentPath = '/'.concat(parentPath);
        if (!parentPath.endsWith('/'))
            parentPath = parentPath.concat('/');
        const isFolder = requestUrl.endsWith('/');
        if (isFolder) {
            return {
                type: 'folder',
                url: decodedUrl,
                name: parsedPath.base,
                path: parsedPath,
                parentPath,
            };
        }
        else {
            return {
                type: 'file',
                url: decodedUrl,
                name: parsedPath.name,
                path: parsedPath,
                parentPath,
            };
        }
    }
    static async getDatabaseItemFromResource(resource, driveDatabaseManager) {
        let databaseResource = null;
        if (resource.type === 'folder') {
            databaseResource = await driveDatabaseManager.findFolderByRelativePath(resource.url);
        }
        if (resource.type === 'file') {
            databaseResource = await driveDatabaseManager.findFileByRelativePath(resource.url);
        }
        return databaseResource?.toItem() ?? null;
    }
    static async setDatabaseItem(type, driveItem, driveDatabaseManager, relativePath) {
        if (type === 'folder') {
            return await driveDatabaseManager.createFolder(driveItem, relativePath);
        }
        if (type === 'file') {
            return await driveDatabaseManager.createFile(driveItem, relativePath);
        }
    }
    static async getDriveItemFromResource(resource, driveFolderService, driveFileService) {
        let item = undefined;
        if (resource.type === 'folder') {
            item = await driveFolderService?.getFolderMetadataByPath(resource.url);
        }
        if (resource.type === 'file') {
            item = await driveFileService?.getFileMetadataByPath(resource.url);
        }
        return item;
    }
    static async getAndSearchItemFromResource({ resource, driveDatabaseManager, driveFolderService, driveFileService, }) {
        let databaseItem = await this.getDatabaseItemFromResource(resource, driveDatabaseManager);
        if (!databaseItem) {
            logger_utils_1.webdavLogger.info('Resource not found on local database', { resource });
            const driveItem = await this.getDriveItemFromResource(resource, driveFolderService, driveFileService);
            if (!driveItem) {
                throw new errors_utils_1.NotFoundError(`Resource not found on Internxt Drive at ${resource.url}`);
            }
            databaseItem = driveItem;
            await this.setDatabaseItem(resource.type, driveItem, driveDatabaseManager, resource.url);
        }
        return databaseItem;
    }
}
exports.WebDavUtils = WebDavUtils;
