"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.MOVERequestHandler = void 0;
const errors_utils_1 = require("../../utils/errors.utils");
const logger_utils_1 = require("../../utils/logger.utils");
const webdav_utils_1 = require("../../utils/webdav.utils");
class MOVERequestHandler {
    dependencies;
    constructor(dependencies) {
        this.dependencies = dependencies;
    }
    handle = async (req, res) => {
        const { driveDatabaseManager, driveFolderService, driveFileService } = this.dependencies;
        const resource = await webdav_utils_1.WebDavUtils.getRequestedResource(req);
        logger_utils_1.webdavLogger.info(`[MOVE] Request received for ${resource.type} at ${resource.url}`);
        const destinationUrl = req.header('destination');
        if (!destinationUrl) {
            throw new errors_utils_1.NotFoundError('[MOVE] Destination folder not received');
        }
        const destinationPath = webdav_utils_1.WebDavUtils.removeHostFromURL(destinationUrl);
        const destinationResource = await webdav_utils_1.WebDavUtils.getRequestedResource(destinationPath);
        logger_utils_1.webdavLogger.info('[MOVE] Destination resource found', { destinationResource });
        const originalDriveItem = await webdav_utils_1.WebDavUtils.getAndSearchItemFromResource({
            resource,
            driveDatabaseManager,
            driveFolderService,
            driveFileService,
        });
        if (destinationResource.path.dir === resource.path.dir) {
            logger_utils_1.webdavLogger.info(`[MOVE] Renaming ${resource.type} with UUID ${originalDriveItem.uuid} to ${destinationResource.name}`);
            const newName = destinationResource.name;
            if (resource.type === 'folder') {
                const folder = originalDriveItem;
                await driveFolderService.renameFolder({
                    folderUuid: folder.uuid,
                    name: newName,
                });
                await driveDatabaseManager.createFolder(folder, destinationResource.url);
            }
            else if (resource.type === 'file') {
                const newType = destinationResource.path.ext.replace('.', '');
                const file = originalDriveItem;
                await driveFileService.renameFile(file.uuid, {
                    plainName: newName,
                    type: newType,
                });
                await driveDatabaseManager.createFile(file, destinationResource.url);
            }
        }
        else {
            logger_utils_1.webdavLogger.info(`[MOVE] Moving ${resource.type} with UUID ${originalDriveItem.uuid} to ${destinationPath}`);
            const destinationFolderResource = await webdav_utils_1.WebDavUtils.getRequestedResource(destinationResource.parentPath);
            const destinationFolderItem = (await webdav_utils_1.WebDavUtils.getAndSearchItemFromResource({
                resource: destinationFolderResource,
                driveDatabaseManager,
                driveFolderService,
            }));
            if (resource.type === 'folder') {
                const folder = originalDriveItem;
                await driveFolderService.moveFolder({
                    folderUuid: folder.uuid,
                    destinationFolderUuid: destinationFolderItem.uuid,
                });
                await driveDatabaseManager.createFolder(folder, destinationPath);
            }
            else if (resource.type === 'file') {
                const file = originalDriveItem;
                await driveFileService.moveFile({
                    fileUuid: file.uuid,
                    destinationFolderUuid: destinationFolderItem.uuid,
                });
                await driveDatabaseManager.createFile(file, destinationPath);
            }
        }
        res.status(204).send();
    };
}
exports.MOVERequestHandler = MOVERequestHandler;
