import { Token, CryptoProvider, Keys, LoginDetails, RegisterDetails, SecurityDetails, TwoFactorAuthQR, RegisterPreCreatedUser, RegisterPreCreatedUserResponse } from './types';
import { UserSettings, UUID } from '../shared/types/userSettings';
import { TeamsSettings } from '../shared/types/teams';
import { ApiSecurity, ApiUrl, AppDetails } from '../shared';
export * from './types';
export declare class Auth {
    private readonly client;
    private readonly appDetails;
    private readonly apiSecurity?;
    private readonly apiUrl;
    static client(apiUrl: ApiUrl, appDetails: AppDetails, apiSecurity?: ApiSecurity): Auth;
    private constructor();
    /**
     * Tries to register a new user
     * @param registerDetails
     */
    register(registerDetails: RegisterDetails): Promise<{
        token: Token;
        user: Omit<UserSettings, 'bucket'> & {
            referralCode: string;
        };
        uuid: UUID;
    }>;
    /**
     * Registers a precreated user
     * @param registerDetails
     * @returns {Promise<RegisterPreCreatedUserResponse>} Returns sign in token, user data and uuid.
     */
    registerPreCreatedUser(registerDetails: RegisterPreCreatedUser): Promise<RegisterPreCreatedUserResponse>;
    /**
     * Requests account unblock email
     *
     * @param {string} email - The email address associated with the account.
     * @returns {Promise<void>} - Resolves when the email is sent.
     */
    requestUnblockAccount(email: string): Promise<void>;
    /**
     * Unblocks account with token
     *
     * @param {string} token - The token received via email to verify and unblock the account.
     * @returns {Promise<void>} - Resolves successfuly when account is unblocked
     */
    unblockAccount(token: string): Promise<void>;
    /**
     * Tries to log in a user given its login details
     * @param details
     * @param cryptoProvider
     */
    login(details: LoginDetails, cryptoProvider: CryptoProvider): Promise<{
        token: Token;
        newToken: Token;
        user: UserSettings;
        userTeam: TeamsSettings | null;
    }>;
    /**
     * Updates the asymmetric keys
     * @param keys
     * @param token
     */
    updateKeys(keys: Keys, token: Token): Promise<unknown>;
    /**
     * Returns general security details
     * @param email
     */
    securityDetails(email: string): Promise<SecurityDetails>;
    /**
     * Generates a new TwoFactorAuth code
     */
    generateTwoFactorAuthQR(): Promise<TwoFactorAuthQR>;
    /**
     * Disables TwoFactorAuthentication
     * @param pass
     * @param code
     */
    disableTwoFactorAuth(pass: string, code: string): Promise<void>;
    /**
     * Store TwoFactorAuthentication details
     * @param backupKey
     * @param code
     */
    storeTwoFactorAuthKey(backupKey: string, code: string): Promise<void>;
    /**
     * Sends request to send the email to delete the account
     * @param email
     */
    sendDeactivationEmail(email: string): Promise<void>;
    /**
     * Confirms the account deactivation
     * @param token
     */
    confirmDeactivation(token: string): Promise<void>;
    /**
     * Checks if the password is correct for this email
     * @param email
     * @param hashedPassword
     * @returns
     */
    areCredentialsCorrect(email: string, hashedPassword: string): Promise<boolean>;
    /**
     * Send email to change password
     * @param email
     */
    sendChangePasswordEmail(email: string): Promise<void>;
    /**
     * Upgrade hash and salt in the database
     * @param newHash
     * @param newSalt
     */
    upgradeHash(newHash: string, newSalt: string): Promise<void>;
    /**
     * Restore password with email link
     * @param token
     * @param password
     * @param salt
     * @param mnemonic
     */
    changePasswordWithLink(token: string | undefined, password: string, salt: string, mnemonic: string): Promise<void>;
    /**
     * Reset account with token
     * @param token
     * @param password
     * @param salt
     * @param mnemonic
     */
    resetAccountWithToken(token: string | undefined, password: string, salt: string, mnemonic: string): Promise<void>;
    private basicHeaders;
    private headersWithToken;
}
